/*******************************************************************************

Copyright (c) Future Technology Devices International

THIS SOFTWARE IS PROVIDED BY FUTURE TECHNOLOGY DEVICES INTERNATIONAL LIMITED "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
FUTURE TECHNOLOGY DEVICES INTERNATIONAL LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES LOSS OF USE, DATA, OR PROFITS OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

FTDI DRIVERS MAY BE USED ONLY IN CONJUNCTION WITH PRODUCTS BASED ON FTDI PARTS.

FTDI DRIVERS MAY BE DISTRIBUTED IN ANY FORM AS LONG AS LICENSE INFORMATION IS NOT MODIFIED.

IF A CUSTOM VENDOR ID AND/OR PRODUCT ID OR DESCRIPTION STRING ARE USED, IT IS THE
RESPONSIBILITY OF THE PRODUCT MANUFACTURER TO MAINTAIN ANY CHANGES AND SUBSEQUENT WHQL
RE-CERTIFICATION AS A RESULT OF MAKING THESE CHANGES.

********************************************************************************

This code is provided as an example only and is not guaranteed by FTDI/BridgeTek.
FTDI/BridgeTek accept no responsibility for any issues resulting from its use.
By using this code, the developer of the final application incorporating any 
parts of this sample project agrees to take full responsible for ensuring its 
safe and correct operation and for any consequences resulting from its use.

Revision History: 
 
Version Author              Date        Description
******* ******************* *********** ****************************************
1.0     G Brown             Feb 2017    Initial version
 
*/

#include <xc.h>
#include "Library.h"
#include "FT8xx.h"
#include "EVE_RGB565.h"
#include "DIGITfont.h"

#define FT_81X_ENABLE
#define LCD_800_480				
#define FT81x_ACTIVE	0x00			


// LCD display parameters
unsigned int lcdWidth;				// Active width of LCD display
unsigned int lcdHeight;				// Active height of LCD display
unsigned int lcdHcycle;				// Total number of clocks per line
unsigned int lcdHoffset;			// Start of active line
unsigned int lcdHsync0;				// Start of horizontal sync pulse
unsigned int lcdHsync1;				// End of horizontal sync pulse
unsigned int lcdVcycle;				// Total number of lines per screen
unsigned int lcdVoffset;			// Start of active screen
unsigned int lcdVsync0;				// Start of vertical sync pulse
unsigned int lcdVsync1;				// End of vertical sync pulse
unsigned char lcdPclk;				// Pixel Clock
unsigned char lcdSwizzle;			// Define RGB output pins
unsigned char lcdPclkpol;			// Define active edge of PCLK

unsigned long ramDisplayList = RAM_DL;		// Set beginning of display list memory
unsigned long ramCommandBuffer = RAM_CMD;	// Set beginning of graphics command memory
unsigned int cmdBufferRd;		// Used to navigate command ring buffer
unsigned int cmdBufferWr = 0x0000;		// Used to navigate command ring buffer
unsigned int cmdOffset = 0x0000;		// Used to navigate command rung buffer
unsigned int point_size = 0x0100;		// Define a default dot size
unsigned long color;				// Variable for changing colors
unsigned char FT81x_GPIO;			// Used for GPIO register
unsigned char dummyRead = 0;
unsigned long TrackerVal = 0;// 32 bit
unsigned char TagVal = 0;
unsigned int SlideVal = 0;

unsigned char dly = 0;
    
    
// #################### Function Declarations ##################################

// Application Layer
void APP_Init(void);
void APP_Calibrate(void);
void APP_FlashingDot(void);
void APP_VertexTranslate(void);                                                
void APP_Text(void);                                                          
void APP_ConvertedBitmap(void);                                                          
void APP_SliderandButton(void);   
void APP_SnapShot2(void);
void APP_DigitsFont(void);

//############################################################################## 
//##############################################################################
//                        MAIN APPLICATION
//##############################################################################  
//##############################################################################

void main(void) 
{
    MCU_Init();    
    APP_Init();
    MCU_UART_Init(); 
    //APP_Calibrate();                                                          // NOTE:  Enable if using any touch demos
    
    // Important Note: Enable only one demo below at a time. 
    // If running any of the following demos, un-comment the APP_Calibrate function above
    //     * APP_SliderAndButton
    
    APP_FlashingDot();                                                        
    
    //APP_VertexTranslate();                                                    

    //APP_Text();                                                               

    //APP_ConvertedBitmap();                                                            
    
    //APP_DigitsFont();
     
    //APP_SliderandButton();                                                    // APP_Calibrate() must be un-commented when using this demo

    while(1)
    {
            // Catch if it ever reaches here
    }    
}

//########################### INITIALISE EVE ###################################  
  
void APP_Init(void)
{
    // ----------------------- Cycle PD pin to reset device --------------------
    
    MCU_PDlow();                                                                // PD low to reset device                                                                

    MCU_Delay_20ms();

    MCU_PDhigh();                                                               // PD high again

    MCU_Delay_20ms();

    // ---------------------- Delay to allow FT81x start-up --------------------    
    
    EVE_CmdWrite(FT81x_ACTIVE, 0x00);                                           // Sends 00 00 00 to wake FT8xx

    MCU_Delay_500ms();                                                          // 500ms delay (EVE requires at least 300ms here))
    
    // --------------- Check that FT8xx ready and SPI comms OK ----------------- 
    
    while (EVE_MemRead8(REG_ID) != 0x7C)                                        // Read REG_ID register (0x302000) until reads 0x7C
    {
    }
      
    while (EVE_MemRead8(REG_CPURESET) != 0x00)                                  // Ensure CPUreset register reads 0 and so FT8xx is ready   
    {
    }

    // ------------------------- Display settings ------------------------------

    // WQVGA display parameters
    lcdWidth   = 800;                                                           // Active width of LCD display
    lcdHeight  = 480;                                                           // Active height of LCD display
    lcdHcycle  = 928;                                                           // Total number of clocks per line
    lcdHoffset = 88;                                                            // Start of active line
    lcdHsync0  = 0;                                                             // Start of horizontal sync pulse
    lcdHsync1  = 48;                                                            // End of horizontal sync pulse
    lcdVcycle  = 525;                                                           // Total number of lines per screen
    lcdVoffset = 32;                                                            // Start of active screen
    lcdVsync0  = 0;                                                             // Start of vertical sync pulse
    lcdVsync1  = 3;                                                             // End of vertical sync pulse
    lcdPclk    = 2;                                                             // Pixel Clock
    lcdSwizzle = 0;                                                             // Define RGB output pins
    lcdPclkpol = 1;                                                             // Define active edge of PCLK
                                          
                                            
    EVE_MemWrite16(REG_HSIZE,   lcdWidth);	
    EVE_MemWrite16(REG_HCYCLE,  lcdHcycle);	
    EVE_MemWrite16(REG_HOFFSET, lcdHoffset);	
    EVE_MemWrite16(REG_HSYNC0,  lcdHsync0);	
    EVE_MemWrite16(REG_HSYNC1,  lcdHsync1);	
    EVE_MemWrite16(REG_VSIZE,   lcdHeight);	
    EVE_MemWrite16(REG_VCYCLE,  lcdVcycle);	
    EVE_MemWrite16(REG_VOFFSET, lcdVoffset);	
    EVE_MemWrite16(REG_VSYNC0,  lcdVsync0);	
    EVE_MemWrite16(REG_VSYNC1,  lcdVsync1);	
    EVE_MemWrite8(REG_SWIZZLE,  lcdSwizzle);	
    EVE_MemWrite8(REG_PCLK_POL, lcdPclkpol);	
  
    FT81x_GPIO = EVE_MemRead8(REG_GPIO);                                        // Read the FT800 GPIO register for a read/modify/write operation
    FT81x_GPIO = FT81x_GPIO | 0x80;                                             // set bit 7 of FT800 GPIO register (DISP) - others are inputs
    EVE_MemWrite8(REG_GPIO, FT81x_GPIO);                                        // Enable the DISP signal to the LCD panel
    
    // Can move these 2 lines to after the first display list to make the start-up appear cleaner to the user
    EVE_MemWrite8(REG_PCLK, lcdPclk);                                           // Now start clocking data to the LCD panel
    EVE_MemWrite8(REG_PWM_DUTY, 127);
    
    // ---------------------- Touch and Audio settings -------------------------

    EVE_MemWrite16(REG_TOUCH_RZTHRESH, 1200);                                   // Eliminate any false touches

    EVE_MemWrite8(REG_VOL_PB, ZERO);                                            // turn recorded audio volume down
    EVE_MemWrite8(REG_VOL_SOUND, ZERO);                                         // turn synthesizer volume down
    EVE_MemWrite16(REG_SOUND, 0x6000);                                          // set synthesizer to mute

    // -------- Initial display list to begin with blank screen ----------------

    ramDisplayList = RAM_DL;                                                    // start of Display List
    EVE_MemWrite32(ramDisplayList, 0x02000000);                                 // Clear Color RGB sets the colour to clear screen to

    ramDisplayList += 4;                                                        // point to next location
    EVE_MemWrite32(ramDisplayList, (0x26000000 | 0x00000007));                  // Clear 00100110 -------- -------- -----CST  (C/S/T define which parameters to clear)

    ramDisplayList += 4;                                                        // point to next location
    EVE_MemWrite32(ramDisplayList, 0x00000000);                                 // DISPLAY command 00000000 00000000 00000000 00000000 (end of display list)

    EVE_MemWrite32(REG_DLSWAP, DLSWAP_FRAME);                                   // Swap display list to make the edited one active
}     
   
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
  
//############################################################################## 
//##############################################################################
//                     INDIVIDUAL DEMO FUNCTIONS
//##############################################################################  
//##############################################################################
    
    
// ############################ TOUCH CALIBRATION ##############################

void APP_Calibrate(void)
{
    cmdOffset = EVE_WaitCmdFifoEmpty();                                         // Wait for command FIFO to be empty and record current position in FIFO
    
    MCU_CSlow();                                                                // CS low begins SPI transaction 
    EVE_AddrForWr(RAM_CMD + cmdOffset);                                         // Send address to which first value will be written

    EVE_Write32(CMD_DLSTART);                                                   // Co-pro starts new DL at RAM_DL + 0
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 // Keep count of bytes sent so that write pointer can be updated at end
                                                                                // Keeping CS low and FT8xx will auto increment address for 'burst write'
    EVE_Write32(CLEAR_COLOR_RGB(0,0,0));                                        // Set the default clear color to black
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

    EVE_Write32(CLEAR(1,1,1));                                                  // Clear the screen Attributes color, stencil and tag buffers
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	
   
    EVE_Write32(CMD_CALIBRATE);                                                 // Calibrate
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	  
        
    EVE_Write32(DISPLAY());                                                     // Instruct the graphics processor to show the list
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

    EVE_Write32(CMD_SWAP);                                                      // Make this list active
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

    MCU_CShigh();                                                               // CS high finishes SPI burst write. Data now loaded into co-pro's FIFO but not yet executed
    
    EVE_MemWrite32(REG_CMD_WRITE, (cmdOffset));                                 // Update circular buffer write pointer to tell FT8xx to execute commands...

    cmdOffset = EVE_WaitCmdFifoEmpty();                                         // ... and wait until co-pro has completed executing them
    // Code will only reach past above line once user has touched the 3 points on the calibration screen                                                                           
    // Code could store result of the calibration (i.e. read the six 32-bit values from registers REG_TOUCH_TRANSFORM_A to _F) in MCU EEPROM and re-load them on each power up 
}
        
  
// ######################## DEMO - FLASHING DOT ##############################
  
void APP_FlashingDot(void)
{
   
    cmdOffset = EVE_WaitCmdFifoEmpty();                                         // Wait for command FIFO to be empty and record current position in FIFO
            
    while(1)
    {   
        if(color == 0x00)                                                       // Toggle colour variable
            color = 0xFF;
        else
            color = 0x00;
        
        MCU_CSlow();                                                            // CS low begins SPI transaction 
        EVE_AddrForWr(RAM_CMD + cmdOffset);                                     // Send address to which first value will be written
       
        EVE_Write32(CMD_DLSTART);                                               // Co-pro starts new DL at RAM_DL + 0
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                             // Keep count of bytes sent so that write pointer can be updated at end
                                                                                // Keeping CS low and FT8xx will auto increment address for 'burst write'

        EVE_Write32(CLEAR_COLOR_RGB(0,0,0));                                    // Set the default clear color to black
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                

        EVE_Write32(CLEAR(1,1,1));                                              // Clear the screen Attributes color, stencil and tag buffers
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                               

        EVE_Write32 (COLOR_RGB(color,0,0));                                     // Specify RGB colour of subsequent items drawn
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 

        EVE_Write32(BEGIN(FTPOINTS));                                           // Begin drawing points
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                               

        EVE_Write32(POINT_SIZE(point_size));                                    // Select the size of the dot to draw
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                              
    
        EVE_Write32(VERTEX2F(100*16,100*16));                                   // Set the point center location
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                               

        EVE_Write32(END());                                                     // End drawing of points
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                           

        EVE_Write32(DISPLAY());                                                 // Instruct the graphics processor to show the list
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                    

        EVE_Write32(CMD_SWAP);                                                  // Make this list active
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                          

        MCU_CShigh();                                                           // Chip Select high concludes burst
    
        EVE_MemWrite32(REG_CMD_WRITE, (cmdOffset));                             // Update the ring buffer pointer 
                                                                                // Co-processor will now execute all of the above commands and create a display list
        cmdOffset = EVE_WaitCmdFifoEmpty();                                     // Await completion of processing and record starting address for next screen update
                                                                                
        MCU_Delay_500ms();                                                      // Delay to slow down the flashing of the dot to allow user to see it
        
        //APP_SnapShot2();
        
    }
}
  
  
// ######################## DEMO - VERTEX TRANSLATE ##########################

void APP_VertexTranslate(void)
  {

    cmdOffset = EVE_WaitCmdFifoEmpty();                                         // Wait for command FIFO to be empty and record current position in FIFO
                    
    MCU_CSlow();                                                            // CS low begins SPI transaction 
    EVE_AddrForWr(RAM_CMD + cmdOffset);                                     // Send address to which first value will be written

    EVE_Write32(CMD_DLSTART);                                               // Co-pro starts new DL at RAM_DL + 0
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                             // Keep count of bytes sent so that write pointer can be updated at end
                                                                            // Keeping CS low and FT8xx will auto increment address for 'burst write'

    EVE_Write32(CLEAR_COLOR_RGB(0,0,0));                                    // Set the default clear color to black
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 

    EVE_Write32(CLEAR(1,1,1));                                              // Clear the screen Attributes color, stencil and tag buffers
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                               

    EVE_Write32(BEGIN(FTPOINTS));                                           // Begin drawing points
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                          

    EVE_Write32(POINT_SIZE(point_size));                                    // Select the size of the dot to draw
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                            


    EVE_Write32 (COLOR_RGB(0xFF,0,0));                                      // Colour = Red
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                  

    EVE_Write32(VERTEX2F(100*16,100*16));                                   // Draw a point at (100,100)
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                



    EVE_Write32 (COLOR_RGB(0,0xFF,0));                                      // Colour = Green
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                              

    EVE_Write32(VERTEX2F(500*16,100*16));                                   // Draw a point at (500,100)
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                



    EVE_Write32 (COLOR_RGB(0,0,0xFF));                                      // Colour = Blue
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                

    EVE_Write32(VERTEX_TRANSLATE_X(250*16));                                // Add an offset of 250 to X direction of subsequent Vertex commands
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                              

    EVE_Write32(VERTEX2F(500*16,100*16));                                   // Draw a point at ((500+250),100) == (750,100))
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                              



    EVE_Write32 (COLOR_RGB(0xFF,0,0xFF));                                   // Colour = Purple
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                

    EVE_Write32(VERTEX2F(500*16,200*16));                                   // Draw a point at ((500+250),200) == (750,200))
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                               



    EVE_Write32 (COLOR_RGB(0xFF,0xFF,0xFF));                                // Colour = White
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                               

    EVE_Write32(VERTEX_TRANSLATE_X(0*16));                                  // Return offset to 0 for subsequent Vertex commands
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                              

    EVE_Write32(VERTEX2F(500*16,200*16));                                   // Draw a point at (500,200)
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                            



    EVE_Write32(END());                                                     // End drawing of points
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                           

    EVE_Write32(DISPLAY());                                                 // Instruct the graphics processor to show the list
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                      

    EVE_Write32(CMD_SWAP);                                                  // Make this list active
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                        

    MCU_CShigh();                                                           // Chip Select high concludes burst

    EVE_MemWrite32(REG_CMD_WRITE, (cmdOffset));                             // Update the ring buffer pointer 
                                                                            // Co-processor will now execute all of the above commands and create a display list
    cmdOffset = EVE_WaitCmdFifoEmpty();                                     // Await completion of processing and record starting address for next screen update

    //APP_SnapShot2();
             
    while(1)
    {
    
    }


}
  

// ############################ DEMO - Text ##################################

void APP_Text(void)
{
    cmdOffset = EVE_WaitCmdFifoEmpty();                                         // Wait for command FIFO to be empty and record current position in FIFO
    
    MCU_CSlow();                                                                // CS low begins SPI transaction 
    EVE_AddrForWr(RAM_CMD + cmdOffset);                                         // Send address to which first value will be written
   
    
    EVE_Write32(CMD_DLSTART);                                                   // Co-pro starts new DL at RAM_DL + 0
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 // Keep count of bytes sent so that write pointer can be updated at end
                                                                                // Keeping CS low and FT8xx will auto increment address for 'burst write'

    EVE_Write32(CLEAR_COLOR_RGB(0,0,0));                                        // Set the default clear color to black
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

    EVE_Write32(CLEAR(1,1,1));                                                  // Clear the screen Attributes color, stencil and tag buffers
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);
    
    EVE_Write32 (COLOR_RGB(255,255,255));                                       // Set colour to white
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4); 
    
    EVE_Write32 (CMD_TEXT);                                                     // command text 0xFFFFFF0C
    EVE_Write16 (100);                                                          // x
    EVE_Write16 (200);                                                          // y        
    EVE_Write16 (25);                                                           // font         
    EVE_Write16 (0);                                                            // options         
    EVE_Write8 (0x48);                                                          // string H 
    EVE_Write8 (0x45);                                                          // string E
    EVE_Write8 (0x4C);                                                          // string L
    EVE_Write8 (0x4C);                                                          // string L
    EVE_Write8 (0x4F);                                                          // string O
    EVE_Write8 (0);                                                             // null 
    EVE_Write8 (0);                                                             // padding
    EVE_Write8 (0);                                                             // padding since overall command not multiple of 4
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 20);                                // Update the command pointer, must be multiple of 4
    
    EVE_Write32(DISPLAY());                                                     // Instruct the graphics processor to show the list
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

    EVE_Write32(CMD_SWAP);                                                      // Make this list active
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

    MCU_CShigh();                                                               // Chip Select high concludes burst
    
    EVE_MemWrite32(REG_CMD_WRITE, (cmdOffset));                                 // Update the ring buffer pointer 
                                                                                // Co-processor will now execute all of the above commands and create a display list
    
    cmdOffset = EVE_WaitCmdFifoEmpty();                                         // Await completion of processing and record starting address for next screen update
        
    //APP_SnapShot2();
    
    while(1)
    {
 
    }
    
    
}

// ############################ DEMO - Bitmap ##################################

/*('file properties: ', 'resolution ', 58, 'x', 75, 'format ', 'ARGB1555', 'stride ', 116, ' total size ', 8700)*/


void APP_ConvertedBitmap(void)
{ 

    unsigned int DataPointer = 0;   
    unsigned int DataSize = 8700;
    unsigned int BitmapDataSize = 0;
    
    // ------------ Load image data -------------

    cmdOffset = EVE_WaitCmdFifoEmpty();                                         // Wait for command FIFO to be empty
    
    DataPointer = 0;
    
    MCU_CSlow();                                                                // CS low begins SPI transaction 
    EVE_AddrForWr(RAM_G);                                                       // Send address to which first value will be written
  
    while(DataPointer < DataSize)
    {
        EVE_Write8(rawData[DataPointer]);                                       // Send data byte-by-byte from array
        DataPointer ++;
    }
    
    BitmapDataSize = DataSize - DataPointer;                                    // Add 3, 2 or 1 bytes padding to make it  a multiple of 4 bytes
    BitmapDataSize = BitmapDataSize & 0x03;                                     // Mask off the bottom 2 bits
    
    if (BitmapDataSize == 0x03)
    {
        EVE_Write8(0x00);
    }
    else if (BitmapDataSize == 0x02)
    {
        EVE_Write8(0x00);
        EVE_Write8(0x00);
    }
    else if (BitmapDataSize == 0x01)
    {
        EVE_Write8(0x00);
        EVE_Write8(0x00);
        EVE_Write8(0x00);
    }
              
    MCU_CShigh();                                                               // CS high after burst write of image data
    
        // ------------ Now create screen to display image -------------
    
    MCU_Delay_20ms();
    
    
    MCU_CSlow();                                                                // CS low begins SPI transaction 
    EVE_AddrForWr(RAM_CMD + cmdOffset);                                         // Send address to which first value will be written
       
    EVE_Write32(CMD_DLSTART);                                                   // Co-pro starts new DL at RAM_DL + 0
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 // Keep count of bytes sent so that write pointer can be updated at end
                                                                                // Keeping CS low and FT8xx will auto increment address for 'burst write'

    EVE_Write32(CLEAR_COLOR_RGB(0,0,0));                                        // Set the default clear color to black
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

    EVE_Write32(CLEAR(1,1,1));                                                  // Clear the screen Attributes color, stencil and tag buffers
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

    EVE_Write32(BITMAP_HANDLE(0));                                              // Set bitmap handle
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	
    
    EVE_Write32(BITMAP_SOURCE(0));                                              // Bitmap data starts at 0
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);
            
    EVE_Write32(BITMAP_LAYOUT(RGB565,116,75));                                  // Tell FT8xx about the properties of the image data
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

    EVE_Write32(BITMAP_SIZE(NEAREST, BORDER, BORDER, 58,75));                   // Tell FT8xx about the on-screen properties of the image 
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	
    
    EVE_Write32(BEGIN(BITMAPS));                                                // Begin drawing bitmaps
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	
    
    EVE_Write32(VERTEX2F(100,100));                                             // Draw at (100,100)
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	
    
    EVE_Write32(END());                                                         // End drawing images
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

    EVE_Write32(DISPLAY());                                                     // Instruct the graphics processor to show the list
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

    EVE_Write32(CMD_SWAP);                                                      // Make this list active
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

    MCU_CShigh();                                                               // Chip Select high concludes burst
    
    EVE_MemWrite32(REG_CMD_WRITE, (cmdOffset));                                 // Update the ring buffer pointer 
                                                                                // Co-processor will now execute all of the above commands and create a display list
    cmdOffset = EVE_WaitCmdFifoEmpty();                                         // Await completion of processing and record starting address for next screen update

    //APP_SnapShot2();
    
    while(1)
    {
 
    }
}



void APP_DigitsFont(void)
{
    unsigned int DataOffset = 1000;
    unsigned int DataPointer = 0;   
    unsigned int DataSize = 7273; //(148+7125))
    unsigned int BitmapDataSize = 0;
    
    // ------------------------- Load image data -------------------------------

    // Load the image data from the MCU's Flash into the RAM_G of the FT81x 
    cmdOffset = EVE_WaitCmdFifoEmpty();                                         // Wait for command FIFO to be empty
    
    MCU_CSlow();                                                                // CS low begins SPI transaction 
    EVE_AddrForWr(RAM_G+DataOffset);                                            // Send address to which first value will be written
  
    while(DataPointer < DataSize)                                               // Stream data
    {
        EVE_Write8(MetricBlock[(DataPointer)]);
        DataPointer ++;
    }
    
   // Add 3, 2 or 1 bytes padding to make it  a multiple of 4 bytes
    BitmapDataSize = DataSize & 0x03;                                           // Mask off the bottom 2 bits
    
    if (BitmapDataSize == 0x03)
    {
        EVE_Write8(0x00);
    }
    else if (BitmapDataSize == 0x02)
    {
        EVE_Write8(0x00);
        EVE_Write8(0x00);
    }
    else if (BitmapDataSize == 0x01)
    {
        EVE_Write8(0x00);
        EVE_Write8(0x00);
        EVE_Write8(0x00);
    }
    
    MCU_CShigh();   
        
    MCU_Delay_20ms();
    
    // ---------- Now create a screen with text written in this font -----------
        
    MCU_CSlow();                                                                // Begin a new SPI burst by setting CS low and sending
    EVE_AddrForWr(RAM_CMD + cmdOffset);                                         // first address to be written
       
    EVE_Write32(CMD_DLSTART);                                                   // DL_Start command tells co-pro to begin new DL
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 // Update the byte count
    
    EVE_Write32(CLEAR_COLOR_RGB(0,0,0));                                        // Set the default clear color to black
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 // Update the command pointer

    EVE_Write32(CLEAR(1,1,1));                                                  // Clear the screen Attributes color, stencil and tag buffers
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 // Update the command pointer
    
    EVE_Write32(BITMAP_HANDLE(14));                                             // Set the bitmap properties for handle 14
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 // Update the command pointer
        
    EVE_Write32(BITMAP_SOURCE(-1252));                                          // This value can be found in font converter output data
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 // Update the command pointer
            
    EVE_Write32(BITMAP_LAYOUT(L1,3,25));                                        // Specify the format, linestride, height 
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 // Update the command pointer

    EVE_Write32(BITMAP_SIZE(NEAREST, BORDER, BORDER, 18,25));                   // Specify bitmap parameters
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 // Update the command pointer
    
    EVE_Write32(CMD_SETFONT);                                                   // Set Font
    EVE_Write32(14);                                                            // Font handle
    EVE_Write32 (1000);                                                         // Address of data in RAM_G
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 12);                                // Update the command pointer
    
    EVE_Write32 (COLOR_RGB(255,255,255));                                       // Specify colour of text
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 // Update the command pointer, must be multiple of 4  
    
    EVE_Write32 (CMD_TEXT);                                                     // command text 0xFFFFFF0C
    EVE_Write16 (100);                                                          // x
    EVE_Write16 (200);                                                          // y        
    EVE_Write16 (14);                                                           // font         
    EVE_Write16 (0);                                                            // options         
    EVE_Write8 (0x30);                                                          // string H 
    EVE_Write8 (0x31);                                                          // string E
    EVE_Write8 (0x32);                                                          // string L
    EVE_Write8 (0x33);                                                          // string L
    EVE_Write8 (0x34);                                                          // string O
    EVE_Write8 (0);                                                             // null 
    EVE_Write8 (0);                                                             // padding
    EVE_Write8 (0);                                                             // padding since overall command not multiple of 4
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 20);                                // Update the command pointer, must be multiple of 4
    
    EVE_Write32(DISPLAY());                                                     // Instruct the graphics processor to show the list
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 // Update the command pointer

    EVE_Write32(CMD_SWAP);                                                      // Make this list active
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                                 // Update the command pointer

    MCU_CShigh();                                                               // Chip select high
    
    EVE_MemWrite32(REG_CMD_WRITE, (cmdOffset));                                 // Update the CMD FIFO pointer
    
    cmdOffset = EVE_WaitCmdFifoEmpty();
    
    //APP_SnapShot2();
    
    while(1)
    {
    }
}




void APP_SnapShot2(void)
{
    unsigned long Pointer = 10000;                                              // Where to start image in RAM_G
    unsigned long SerialCounter = 0;
    unsigned int ARGB = 0;
    unsigned char SerByteAR = 0;
    unsigned char SerByteGB = 0;
    unsigned char StartCode = 0;
        
    cmdOffset = EVE_WaitCmdFifoEmpty();                                         // Await co-processor idle
    
    MCU_CSlow();                                                                // CS low
    EVE_AddrForWr(RAM_CMD + cmdOffset);                                         // Address for writing command
    EVE_Write32 (CMD_SNAPSHOT2);                                                // Snapshot 2 command
    EVE_Write32 (6);                                                            // Format ARGB4                                                    
    EVE_Write32 (Pointer);                                                      // Pointer - where to store data in RAM
    EVE_Write16 (0);                                                            // top-left coordinate x
    EVE_Write16 (0);                                                            // top-left coordinate y                                                                  
    EVE_Write16 (800);                                                          // lower-right coordinate x    
    EVE_Write16 (480);                                                          // lower-right coordinate y
    MCU_CShigh();          
    
    cmdOffset = EVE_IncCMDOffset(cmdOffset, 20);                                // Keep track of command size
                          
    EVE_MemWrite32(REG_CMD_WRITE, (cmdOffset));                                 // Set reg_cmd_write to execute commands
    
    cmdOffset = EVE_WaitCmdFifoEmpty();
    
    MCU_Delay_500ms();                                                          // ensure co-pro has finished taking snapshot

    // NOTE: Size will be 2 * 800 * 480 = 768000
        
    
    //Wait here until we receive starting code 0x53 over UART
    StartCode = 0x00;
    while(StartCode != 0x53)
    {
        StartCode = MCU_UART_Rx();
    }
    
    // Loop round, reading 2 bytes each time from RAM_G which contain the ARGB data for each pixel.
    // The first byte contains 4 bits of alpha and 4 bits of red data. 
    // The second byte contains 4 bits of green and 4 bits of blue data.
    // These bytes are sent to the UART.
    // Here, the first value read is at RAM_G = 10000 as denoted by 'Pointer' which 
    // was where the SnapShot command specified the image data to start
    // SerialCounter will go from 0 up to 768999 to read 768000 bytes (or 384000 16-bit values) in total
    
    
    SerialCounter = 0;
    while(SerialCounter != 768000)
    {
        ARGB = EVE_MemRead16(SerialCounter + Pointer);
        
        SerByteAR = (unsigned char)(ARGB >> 8);
        SerByteGB = (unsigned char)(ARGB);
        
        MCU_UART_Tx(SerByteAR);  
        MCU_UART_Tx(SerByteGB);
       
        SerialCounter = SerialCounter + 2; // increment address to read by 2
    }
            
    while(1)
    {
        Nop();
    }
            
}

 


void APP_SliderandButton(void)
{
    unsigned int Button3D = 0;
    
    cmdOffset = EVE_WaitCmdFifoEmpty();                                         // Wait for command FIFO to be empty and record current position in FIFO
            
    while(1)
    {
        MCU_CSlow();                                                            // CS low begins SPI transaction 
        EVE_AddrForWr(RAM_CMD + cmdOffset);                                     // Send address to which first value will be written
       
        EVE_Write32(CMD_DLSTART);                                               // Co-pro starts new DL at RAM_DL + 0
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                             // Keep count of bytes sent so that write pointer can be updated at end
                                                                                // Keeping CS low and FT8xx will auto increment address for 'burst write'

        EVE_Write32(CLEAR_COLOR_RGB(0,0,0));                                    // Set the default clear color to black
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                             

        EVE_Write32(CLEAR(1,1,1));                                              // Clear the screen Attributes color, stencil and tag buffers
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	
            
        
        EVE_Write32(TAG_MASK(1));                                               // Enable tagging
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

        
        EVE_Write32(TAG(2));                                                    // Tag following items with tag '2' 
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

        EVE_Write32 (CMD_BUTTON);                                               // command button 0xFFFFFF0D
        EVE_Write16 (400);                                                      // x
        EVE_Write16 (200);                                                      // y        
        EVE_Write16 (80);                                                       // w         
        EVE_Write16 (30);                                                       // h         
        EVE_Write16 (26);                                                       // font 
        EVE_Write16 (Button3D);                                                 // options 0 = 3D effect or 256 = flat
        EVE_Write8 (0x42);                                                      // string B
        EVE_Write8 (0x75);                                                      // string u
        EVE_Write8 (0x74);                                                      // string t
        EVE_Write8 (0x74);                                                      // string t
        EVE_Write8 (0x6F);                                                      // string o     
        EVE_Write8 (0x6E);                                                      // string n 
        EVE_Write8 (0);                                                         // null terminates string 
        EVE_Write8 (0);                                                         // pad with extra zero to make multiple of 4 in total
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 24);                            // Update the command pointer, must be multiple of 4
               
        
        EVE_Write32(TAG(5));                                                    // Tag following items with tag '5'
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	
    
        EVE_Write32 (CMD_SLIDER);                                               // command slider 0xFFFFFF10
        EVE_Write16 (0x00F0);                                                   // x
        EVE_Write16 (0x0028);                                                   // y        
        EVE_Write16 (0x000F);                                                   // width         
        EVE_Write16 (0x00B0);                                                   // height         
        EVE_Write16 (0x0000);                                                   // options 
        EVE_Write16 (SlideVal);                                                 // value (i.e. position of handle) 
        EVE_Write16 (0x00FF);                                                   // range of slider
        EVE_Write16 (0x0000);                                                   // dummy
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 20);                            // Update the command pointer, must be multiple of 4

        EVE_Write32 (CMD_TRACK);                                                // command track 0xFFFFFF2C
        EVE_Write16 (0x00F0);                                                   // x
        EVE_Write16 (0x0028);                                                   // y        
        EVE_Write16 (0x000F);                                                   // width         
        EVE_Write16 (0x00B0);                                                   // height         
        EVE_Write16 (0x0005);                                                   // tag the tracked area with 5
        EVE_Write16 (0x0000);                                                   // dummy
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 16);                            // Update the command pointer, must be multiple of 4    



        EVE_Write32(TAG_MASK(0));                                               // Tag mask - disable tagging of any subsequent items
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

        EVE_Write32(BEGIN(FTPOINTS));                                           // Begin drawing points
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

        EVE_Write32(POINT_SIZE(point_size));                                    // Select the size of the dot to draw
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);

        EVE_Write32 (COLOR_RGB(color,0,0));                                       // Colour = red or black (black == invisible since matches background colour)
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);                           

        EVE_Write32(VERTEX2F(440*16,100*16));                                   // Set the point center location (400+ half width of button))
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

        EVE_Write32(END());                                                     // End the point
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

        EVE_Write32(DISPLAY());                                                 // Instruct the graphics processor to show the list
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

        EVE_Write32(CMD_SWAP);                                                  // Make this list active
        cmdOffset = EVE_IncCMDOffset(cmdOffset, 4);	

        MCU_CShigh();                                                           // Chip Select high concludes burst

        EVE_MemWrite32(REG_CMD_WRITE, (cmdOffset));                             // Update the ring buffer pointer 
                                                                                // Co-processor will now execute all of the above commands and create a display list
        cmdOffset = EVE_WaitCmdFifoEmpty();                                     // Await completion of processing and record starting address for next screen update
        
        // ------ read tag and tracker values --------
        TagVal = EVE_MemRead8(REG_TOUCH_TAG);                                   // Get Tag value
        TrackerVal = EVE_MemRead32(REG_TRACKER);                                // Read the value of the tag and track register

        if(TagVal == 2)   
        {// If button pushed tag register will read 2
            color = 0xFF;                                                       // change red amount to 255
            Button3D = 256;
        }
        else  
        {// Otherwise...
            color = 0x00;                                                       // change red amount to 0
            Button3D = 0;
        }
        
        if(TagVal == 5)                                                         // if slider touched...
        {
            SlideVal = (TrackerVal >> 24);                                      // ... then get the tracker value. 
            // Note: Value of tracking is 16 bits but we only want upper 8 bits since the slider is set for 8 bit resolution
        }
                
        //APP_SnapShot2();
        
        // Result of button press and slider value will be displayed next time round
    }
} 




























